// Disclaimer:
// This software is provided AS IS for educational purposes only,
// with no warranties, and confers no rights. Error checking is 
// elided for clarity. Carry out sufficient tests prior to using
// it in a productive environment.
// CASIO Europe GmbH accepts no responsibility for loss or damage
// arising from its use, including damage from virus.
//
// Please, note: This configuration information may be changed
// without any further notice.
// Casio Europe GmbH does not guarantee future compatibility. By 
// using this software you agree to these terms and conditions.
//
// Version 2.05 - May 11, 2012
// Supported Devices: IT-300, IT-600, IT-800, DT-X7, DT-X8, DT-X11, 
// and DT-X30.
//
// Remarks: You need latest versions of patches and runtime
// libraries be installed onto your mobile device.
//////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Globalization;
using Calib; // -> CASIO Libraries

namespace ScannerDemo
{
    public partial class OBRSampleForm : Form
    {
        #region Fields

        /// <summary>
        /// Receiver for notifications from scanner.
        /// </summary>
        private OBRMessageWindow obrMessageWindow = null;
        /// <summary>
        /// Target platform.
        /// </summary>
        private int modelName = 0;
        /// <summary>
        /// Default value for DT-X7.
        /// </summary>
        private int mouseEmulation = SystemLibNet.Def.EMUMOUSE_DISABLE;
        /// <summary>
        /// Default value of Fn key operation (used for DT-X7).
        /// </summary>
        private int fnOperation = SystemLibNet.Def.STATE_FN_NON;
        /// <summary>
        /// Hot key to quit.
        /// </summary>
        private int exitKey = (int)Keys.Back;
        /// <summary>
        /// Exit button for DT-X30CE.
        /// </summary>
        private const int KeysHyphen = 189;

        #endregion

        #region OBRSampleForm

        /// <summary>
        /// Initializes a new instance of the <see cref="OBRSampleForm"/> class.
        /// </summary>
        public OBRSampleForm()
        {
            InitializeComponent();

            Resources.Culture = CultureInfo.CurrentCulture;
            Text = Resources.MainText;
            this.lblDataDescription.Text = Resources.LabelData;
            this.lblTypeDescription.Text = Resources.LabelType;
            this.lblHelp.Text = Resources.LabelHelp;
            this.btnExit.Text = Resources.ButtonExit;
            
            this.obrMessageWindow = new OBRMessageWindow(this);

            this.modelName = GetModelName();

            switch (this.modelName)
            {
                case 7:     // IT-600
                    this.exitKey = (int)Keys.Back;
                    this.lblQuitInfo.Text = Resources.LabelQuitInfo;
                    break;
                case 8:     // DT-X11
                    this.exitKey = (int)Keys.Delete;
                    this.lblQuitInfo.Text = Resources.LabelQuitInfo;
                    break;
                case 9:     // DT-X7
                    this.exitKey = (int)Keys.F4;
                    this.btnExit.BackColor = Color.Yellow;
                    EnableDTX7Mode();
                    break;
                case 10:    // DT-X30 Windows Embedded CE
                    this.exitKey = KeysHyphen;
                    this.btnExit.BackColor = Color.Yellow;
                    break;
                case 11:    // DT-X30 Windows Mobile
                    this.exitKey = (int)Keys.F4;
                    this.btnExit.BackColor = Color.Yellow;
                    break;
                case 13:    // IT-800 Windows Mobile
                    this.exitKey = (int)Keys.F4;
                    this.btnExit.BackColor = Color.Yellow;
                    break;
                case 15:    // IT-300
                    this.exitKey = (int)Keys.F4;
                    this.btnExit.BackColor = Color.Yellow;
                    break;
                case 16:    // DT-X8
                    this.exitKey = (int)Keys.F4;
                    this.btnExit.BackColor = Color.Yellow;
                    break;
                default:
                    break;
            }

            OpenScanner();
        }

        /// <summary>
        /// Handles the Closing event of the OBRSampleForm control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
        private void OBRSampleForm_Closing(object sender, CancelEventArgs e)
        {
            if (this.modelName == 9)
            {
                DisableDTX7Mode();
            }
            CloseScanner();
        }

        /// <summary>
        /// Handles the Click event of the btnExit control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// Handles the KeyDown event of the OBRSampleForm control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.KeyEventArgs"/> instance containing the event data.</param>
        private void OBRSampleForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyValue == this.exitKey)
            {
                Close();
            }
        }

        /// <summary>
        /// Enables special mode for DT-X7 that 
        /// switches the mouse emulation off.
        /// </summary>
        /// <remarks>Mouse emulation would block right trigger key.</remarks>
        private void EnableDTX7Mode()
        { 
            int result;

            result = SystemLibNet.Api.SysGetEmulateMouseState(
                ref this.mouseEmulation);
            result = SystemLibNet.Api.SysSetEmulateMouseState(
                SystemLibNet.Def.EMUMOUSE_DISABLE);

            result = SystemLibNet.Api.SysGetFnKeyOperation();

            if (result != SystemLibNet.Def.FUNCTION_UNSUPPORT)
            {
                this.fnOperation = result;

                result = SystemLibNet.Api.SysSetFnKeyOperation(
                    this.fnOperation | SystemLibNet.Def.STATE_FN_4);
            }
        }

        /// <summary>
        /// Disables special mode for DT-X7.
        /// </summary>
        private void DisableDTX7Mode()
        {
            int result;

            result = SystemLibNet.Api.SysSetEmulateMouseState(
                this.mouseEmulation);

            result = SystemLibNet.Api.SysSetFnKeyOperation(
                this.fnOperation);
        }

        #endregion

        #region Open Scanner

        /// <summary>
        /// Opens the scanner.
        /// </summary>
        private void OpenScanner()
        {
            int result;

            string path = GetIniFilePath();
            if (File.Exists(path) == true)
            {
                // Read settings from file.
                result = OBReadLibNet.Api.OBRLoadConfigFile();
                // Activate settings from file.
                result = OBReadLibNet.Api.OBRSetDefaultSymbology();
            }

            // Select notification by window message.
            result = OBReadLibNet.Api.OBRSetScanningNotification(
                OBReadLibNet.Def.OBR_MESSAGE,
                this.obrMessageWindow.Hwnd);
            // Set output to buffer.
            result = OBReadLibNet.Api.OBRSetBuffType(OBReadLibNet.Def.OBR_BUFOBR);
            // Select continuous scanning while trigger key is pressed.
            result = OBReadLibNet.Api.OBRSetScanningType(OBReadLibNet.Def.OBR_TYPE1);

            result = MakeParticularSettings();

            result = OBReadLibNet.Api.OBROpen(IntPtr.Zero, 0);
        }

        #endregion

        #region Close Scanner

        private void CloseScanner()
        {
            int result;

            result = OBReadLibNet.Api.OBRClose();
        }

        #endregion

        #region Decoding

        /// <summary>
        /// Reads data from buffer.
        /// </summary>
        public void Decode()
        {
            int result;
            int size = 0;
            byte number = 0;
            int code = 0;
            byte[] buffer = new byte[0xFF];
            byte length = (byte)buffer.Length;

            // Is there any record in the OBR buffer?
            result = OBReadLibNet.Api.OBRGetStatus(ref size, ref number);

            if (number > 0)
            {
                result = OBReadLibNet.Api.OBRGets(buffer, ref code, ref length);

                if ( result == OBReadLibNet.Def.OBR_OK)
                {
                    ASCIIEncoding encoder = new ASCIIEncoding();
                    lblData.Text = encoder.GetString(buffer, 0, buffer.Length).Trim();
                    lblType.Text = GetType(code);
                }
            }

            // Do some error handling here...

            // Clear buffer in any case.
            result = OBReadLibNet.Api.OBRClearBuff();
        }

        #endregion

        #region Code Type

        /// <summary>
        /// Gets the type.
        /// </summary>
        /// <param name="code">The code ID.</param>
        /// <returns></returns>
        private string GetType(int code)
        {
            string result;

            switch (code)
            {
                case OBReadLibNet.Def.OBR_CD39:
                    result = "Code 39";
                    break;
                case OBReadLibNet.Def.OBR_NW_7:
                    result = "NW-7";
                    break;
                case OBReadLibNet.Def.OBR_WPCA:
                    result = "WPC Addon";
                    break;
                case OBReadLibNet.Def.OBR_WPC:
                    result = "WPC";
                    break;
                case OBReadLibNet.Def.OBR_UPEA:
                    result = "UPC-E Addon";
                    break;
                case OBReadLibNet.Def.OBR_UPE:
                    result = "UPC-E";
                    break;
                case OBReadLibNet.Def.OBR_IDF:
                    result = "Industrial 2of5";
                    break;
                case OBReadLibNet.Def.OBR_ITF:
                    result = "Interleaved 2of5";
                    break;
                case OBReadLibNet.Def.OBR_CD93:
                    result = "Code 93";
                    break;
                case OBReadLibNet.Def.OBR_CD128:
                    result = "Code 128/GS1-128";
                    break;
                case OBReadLibNet.Def.OBR_MSI:
                    result = "MSI";
                    break;
                case OBReadLibNet.Def.OBR_IATA:
                    result = "IATA";
                    break;
                case OBReadLibNet.Def.OBR_RSS14:
                    result = "GS1 DataBar";
                    break;
                case OBReadLibNet.Def.OBR_RSSLTD:
                    result = "GS1 DataBar Limited";
                    break;
                case OBReadLibNet.Def.OBR_RSSEXP:
                    result = "GS1 DataBar Expanded";
                    break;
                default:
                    result = "Unhandled Type";
                    break;
            }

            return result;
        }

        #endregion

        #region Settings

        /// <summary>
        /// Makes particular settings.
        /// </summary>
        /// <returns>Success of operation as a return code from OBReadLibNet.</returns>
        private int MakeParticularSettings()
        {
            int result;

            result = OBReadLibNet.Api.OBRSetCode39Option(
                OBReadLibNet.Def.OBR_CODE_ENABLE,
                2,
                30,
                OBReadLibNet.Def.OBR_39ASOFF,
                OBReadLibNet.Def.OBR_CHKDOFF,
                OBReadLibNet.Def.OBR_CHKKON);

            return result;
        }

        #endregion

        #region System Library

        /// <summary>
        /// Gets the name of the model.
        /// </summary>
        /// <returns></returns>
        private int GetModelName()
        {
            int result;
            int dwModel = 0;
            int dwVersion = 0;
            int dwPlatform = 0;

            result = SystemLibNet.Api.SysGetModelName(
                ref dwModel,
                ref dwVersion,
                ref dwPlatform);

            return dwModel;
        }

        private string GetIniFilePath()
        {
            int result;
            char[] buffer = new char[256];
            string path = "";

            result = SystemLibNet.Api.SysGetSystemName(
                SystemLibNet.Def.OBRIni_FilePath, buffer);

            if (result == -1)
            {
                path = new string(buffer);
                path = path.TrimEnd('\0');
            }

            return path;
        }

        #endregion
    }
}